@@ -1,18 +1,25 @@
 from __future__ import print_function
+
 from future import standard_library
+
 standard_library.install_aliases()
 from builtins import object
+
 import astropy.units as u
 import pytest
-from astromodels.functions import Uniform_prior, Log_uniform_prior
-from astromodels.functions import  Powerlaw
+
+from astromodels.functions import Log_uniform_prior, Powerlaw, Uniform_prior
 
 __author__ = 'giacomov'
 
-from astromodels.core.parameter import Parameter, SettingOutOfBounds, \
-    CannotConvertValueToNewUnits, NotCallableOrErrorInCall, IndependentVariable, ParameterMustHaveBounds
-from astromodels.functions import Line
+from astromodels.core.parameter import (CannotConvertValueToNewUnits,
+                                        IndependentVariable,
+                                        NotCallableOrErrorInCall, Parameter,
+                                        ParameterMustHaveBounds,
+                                        SettingOutOfBounds, turn_off_parameter_transforms)
 from astromodels.core.parameter_transformation import LogarithmicTransformation
+from astromodels.functions import Line
+from astromodels.utils.configuration import astromodels_config
 
 def test_default_constructor():
 
@@ -98,8 +105,12 @@ def test_constructor_complete():
 
 def test_constructor_with_transform():
 
+    # make sure our local config does not screw this up
+    
+    astromodels_config.modeling.use_parameter_transforms = True
+    
     p = Parameter('test_parameter', 1.0, min_value=0.1, max_value=5.0, delta=0.2, desc='test',
-                  free=False, unit=u.MeV, prior=Uniform_prior(), is_normalization=True,)
+                  free=False, unit=u.MeV, prior=Uniform_prior(), is_normalization=True, transformation=LogarithmicTransformation())
 
     assert p.min_value == 0.1
     assert p.max_value == 5.0
@@ -115,10 +126,46 @@ def test_constructor_with_transform():
 
     assert p.is_normalization
 
-    assert p.has_transformation
+    assert p.has_transformation()
+
+    p.remove_transformation()
+
+    assert not p.has_transformation()
+
+    p.restore_transformation()
+
+    assert p.has_transformation()
+
+    assert p.transformation == p._original_transformation
+
     
     p.display()
 
+    astromodels_config.modeling.use_parameter_transforms = False
+
+    assert p.transformation is None
+    assert not p.has_transformation()
+
+    astromodels_config.modeling.use_parameter_transforms = True
+
+    assert p.transformation is not None
+    assert p.has_transformation()
+
+    
+    with turn_off_parameter_transforms():
+
+        assert p.transformation is None
+        assert not p.has_transformation()
+
+        assert not astromodels_config.modeling.use_parameter_transforms
+
+    assert p.transformation is not None
+    assert p.has_transformation()
+        
+    assert astromodels_config.modeling.use_parameter_transforms
+    
+
+    
 
     
 def test_conflicting_units_in_initial_value_and_unit_keyword():
@@ -396,7 +443,7 @@ def test_set_auxiliary_variable():
 
     p1.add_auxiliary_variable(x, law)
 
-    assert p1.has_auxiliary_variable() == True
+    assert p1.has_auxiliary_variable == True
 
     assert p1.value == 3.0
 
@@ -446,7 +493,7 @@ def test_remove_auxiliary_variable():
 
     p1.remove_auxiliary_variable()
 
-    assert p1.has_auxiliary_variable() == False
+    assert p1.has_auxiliary_variable == False
 
     p1.value = -1.0
 
@@ -702,7 +749,7 @@ def test_links_and_pickle():
 
     p = pickle.loads(d)
 
-    assert p.has_auxiliary_variable() == True
+    assert p.has_auxiliary_variable == True
 
     assert p.value == 3.0
 
